<?php
namespace Teknisa\PLA\Modules\Cardapio\Services;

use Doctrine\ORM\EntityManager;

class Pla00160AjustarComensaisEstimadosCardapioService {

    protected $entityManager;
	
	public function __construct($entityManager) {
		$this->entityManager = $entityManager;
	}

    public function verificaNecessidadeJustificativa(){
        $query = "SELECT QTDIABLOQCAR, IDJUSTALTCOMENS
                  FROM PARAMGERAL
                 WHERE CDPARGERAL = :CDPARGERAL";
        $params = array(
            "CDPARGERAL"   => '0001'
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    public function retornaDiasCardapioBloqueado($dtInicial, $dtFinal, $nrLista){
        $query = "SELECT COUNT(1) AS EXISTE_CARD_BLOQUEADO
                  FROM CARDADIA C
                       LEFT JOIN PARAMGERAL PG ON PG.CDPARGERAL = '00001'
                       LEFT JOIN PARAMFILIAL PF ON PF.CDFILIAL = C.CDFILIAL
                 WHERE C.DTCARDAPIO BETWEEN :P_DTINICIAL AND :P_DTFINAL
                   AND C.DTCARDAPIO < NVL(PF.DTPROCESSA, TRUNC(SYSDATE)) + NVL(PG.QTDIABLOQCAR, 0)
                   AND EXISTS (SELECT 1 FROM LISTA L WHERE L.NRLISTA = :NRLISTA AND L.CDLISTA = C.CDFILIAL||'_'||C.CDSERVFILI)";
        $params = array(
            "P_DTINICIAL"   => $dtInicial,
            "P_DTFINAL"   => $dtFinal,
            "NRLISTA"   => $nrLista
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    public function retornaCardapios($dtInicial, $dtFinal, $nrLista){
        $query = "SELECT C.CDFILIAL, C.CDSERVFILI, C.NRCARDAPIO, TO_CHAR(C.DTCARDAPIO, 'DD/MM/YYYY') AS DTCARDAPIO, 
                    C.QTESTITURNO1, C.QTESTITURNO2, C.QTESTITURNO3, C.QTESTITURNO4, C.QTCLIESTICAR, C.QTCOMESTICAR,
                    C.QTFUNESTICAR,C.QTDOAESTICAR
                  FROM CARDADIA C
                 WHERE C.DTCARDAPIO BETWEEN :P_DTINICIAL AND :P_DTFINAL
                   AND EXISTS (SELECT 1 FROM LISTA L WHERE L.NRLISTA = :NRLISTA AND L.CDLISTA = C.CDFILIAL||'_'||C.CDSERVFILI)";
        $params = array(
            "P_DTINICIAL"   => $dtInicial,
            "P_DTFINAL"   => $dtFinal,
            "NRLISTA"   => $nrLista
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    public function retornaCardacli($cdfilial, $nrcardapio){
        $query = "SELECT * FROM CARDACLI
                WHERE CDFILIAL = :CDFILIAL
                    AND NRCARDAPIO = :NRCARDAPIO";
        $params = array(
            "CDFILIAL"   => $cdfilial,
            "NRCARDAPIO"   => $nrcardapio
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    public function updateCardadia($dados) {
        $query = "UPDATE CARDADIA SET QTCOMESTICAR = :QTCOMESTICAR, 
                                      QTCLIESTICAR = :QTCLIESTICAR,
                                      QTESTITURNO1 = :QTESTITURNO1,
                                      QTESTITURNO2 = :QTESTITURNO2,
                                      QTESTITURNO3 = :QTESTITURNO3,
                                      QTESTITURNO4 = :QTESTITURNO4
                 WHERE  DTCARDAPIO = :DTCARDAPIO
                        AND CDFILIAL = :CDFILIAL
                        AND CDSERVFILI = :CDSERVFILI
                        AND NRCARDAPIO = :NRCARDAPIO";
        $sql = $this->entityManager->getConnection()->prepare($query);
        $params = array(
            "QTCOMESTICAR" => $dados['QTCOMESTICAR'],
            "QTCLIESTICAR" => $dados['QTCLIESTICAR'],
            "QTESTITURNO1" => $dados['QTESTITURNO1'],
            "QTESTITURNO2" => $dados['QTESTITURNO2'],
            "QTESTITURNO3" => $dados['QTESTITURNO3'],
            "QTESTITURNO4" => $dados['QTESTITURNO4'],
            "DTCARDAPIO" => $dados['DTCARDAPIO'],
            "CDFILIAL" => $dados['CDFILIAL'],
            "CDSERVFILI" => $dados['CDSERVFILI'],
            "NRCARDAPIO" => $dados['NRCARDAPIO']
        );
        $sql->execute($params);
    }
    
    public function updateCardacli($dados) {
        $query = "UPDATE CARDACLI SET QTCOMESTICLI = :QTCOMESTICLI
                WHERE CDFILIAL = :CDFILIAL
                      AND NRCARDAPIO = :NRCARDAPIO
                      AND CDCLIENTE = :CDCLIENTE
                      AND CDDESTFILI = :CDDESTFILI";
        $sql = $this->entityManager->getConnection()->prepare($query);
        $params = array(
            "QTCOMESTICLI" => $dados['QTCOMESTICLI'],
            "CDFILIAL" => $dados['CDFILIAL'],
            "NRCARDAPIO" => $dados['NRCARDAPIO'],
            "CDCLIENTE" => $dados['CDCLIENTE'],
            "CDDESTFILI" => $dados['CDDESTFILI']
        );
        $sql->execute($params);
    }
    
    public function insertJustificativa($dados) {
        $query = "INSERT INTO JUSTALTERCOMENS (NRJUSTALTERCOMENS, CDFILIAL, CDSERVFILI, DTCARDAPIO, QTCOMENSTOTOLD, QTCOMENSTOTNEW,
                             DTHRALTERCOMENS, CDOPERADOR, DSJUSTIFICATIVA, DSACAO, IDORIGALTERACAO)
                VALUES (:NRJUSTALTERCOMENS, :CDFILIAL, :CDSERVFILI, :DTCARDAPIO, :QTCOMENSTOTOLD, :QTCOMENSTOTNEW,
                        :DTHRALTERCOMENS, :CDOPERADOR, :DSJUSTIFICATIVA, :DSACAO, 'AJUSTARECOMENS')";
        $sql = $this->entityManager->getConnection()->prepare($query);
        $params = array(
            "NRJUSTALTERCOMENS" => $dados['NRJUSTALTERCOMENS'],
            "CDFILIAL" => $dados['CDFILIAL'],
            "CDSERVFILI" => $dados['CDSERVFILI'],
            "DTCARDAPIO" => $dados['DTCARDAPIO'],
            "QTCOMENSTOTOLD" => $dados['QTCOMENSTOTOLD'],
            "QTCOMENSTOTNEW" => $dados['QTCOMENSTOTNEW'],
            "DTHRALTERCOMENS" => $dados['DTHRALTERCOMENS'],
            "CDOPERADOR" => $dados['CDOPERADOR'],
            "DSJUSTIFICATIVA" => $dados['DSJUSTIFICATIVA'],
            "DSACAO" => $dados['DSACAO']
        );
        $sql->execute($params);
    }
    
}