<?php
namespace Teknisa\PLA\Modules\Medicao\Controllers;

use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DTO\Request\DataSet;
use \Zeedhi\Framework\DTO\Response;
use \Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DataSource;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class Pla10200RetiradaPlanejamentoInventarioController extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $crudService;
    protected $parameterBag;
    protected $newCode;
    protected $connection;
    protected $environment;
    protected $retiradaInventarioService;
    protected $dataSourceName = 'pla10200_retirada_planejamento_inventario';
    protected $libUtilities;

    public function __construct($crudService, $parameterBag, $newCode, $connection, $environment, $retiradaInventarioService,$libUtilities) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->newCode = $newCode;
        $this->connection = $connection;
        $this->environment = $environment;
        $this->retiradaInventarioService = $retiradaInventarioService;
        $this->libUtilities  = $libUtilities;
    }
    public function getLancamentoProduto(Request\Filter $request, Response $response){
        $this->dataSourceName = 'pla10200_lancamento_produto';
        parent::find($request, $response);
    }

    public function getRetiradaPlanejamentoInventario(Request\Filter $request, Response $response){
        try{
            $dataSourceName = 'pla10200_retirada_planejamento_inventario';
            parent::find($request, $response);
        } catch(\Exception $e){
            $response->setError($e->getMessage());
        }
    }

    public function isValidUnit(Request\Row $request, Response $response){
        $CDFILIAL_FILTRADA = $request->getRow()['CDFILIAL'];
        $params = $this->getParamFilial($CDFILIAL_FILTRADA);
        $response->addDataSet(new DataSource\DataSet('paramsUnit', $params));
    }

    public function verifyControlEstoqProd(Request\Row $request, Response $response){
        $row = $request->getRow();
        $query = 'SELECT PRODFILI.IDCNTRESTOQ
                        FROM PRODFILI, PRODUTO
                    WHERE (PRODFILI.CDFILIAL = :CDFILIAL)
                        AND (PRODUTO.CDPRODUTO = :CDPRODUTO)
                        AND (PRODFILI.CDPRODUTO = PRODUTO.CDPRODESTO)';
        $params = array(
            'CDFILIAL' => $row['CDFILIAL'],
            'CDPRODUTO' => $row['CDPRODUTO']
        );
        $pstmt = $this->connection->prepare($query);
        $pstmt->execute($params);
        $result = $pstmt->fetchAll();
        $result = empty($result) ? array() : $result[0];
        $response->addDataSet(new DataSource\DataSet('isControlEstoq', $result));
        if(isset($row['NRINVRETIPLA'])){
            $this->isDuplicatedProduct($request, $response);
        }
    }

    public function isDuplicatedProduct(Request\Row $request, Response $reponse){
        $row = $request->getRow();
        $query = "SELECT IT.CDPRODUTO FROM INVENTRETIPLA INV, ITEMINVRETIPLA IT, PRODUTO PD WHERE INV.CDFILIAL = :CDFILIAL AND INV.NRINVRETIPLA = :NRINVRETIPLA AND INV.CDFILIAL = IT.CDFILIAL AND INV.NRINVRETIPLA = IT.NRINVRETIPLA AND PD.CDPRODUTO = IT.CDPRODUTO AND IT.CDPRODUTO = :CDPRODUTO";
        $pstmt = $this->connection->prepare($query);
        $params = array(
            'CDFILIAL' => $row['CDFILIAL'],
            'CDPRODUTO' => $row['CDPRODUTO'],
            'NRINVRETIPLA' => $row['NRINVRETIPLA']
        );
        $pstmt->execute($params);
        $reponse->addDataSet(new DataSource\DataSet('alreadyExists', $pstmt->fetchAll()));
    }

    private function getParamFilial ($CDFILIAL_FILTRADA){
        $query = "SELECT IDPERMESTNEG, IDUTILLOTE, IDUTILALMOX FROM PARAMFILIAL WHERE CDFILIAL = :CDFILIAL";
        $pstmt = $this->connection->prepare($query);
        $params = array(
            'CDFILIAL' => $CDFILIAL_FILTRADA
        );
        $pstmt->execute($params);
        return $pstmt->fetchAll();
    }

    private function insereRomaneio($row){
        $query = "INSERT INTO INVENTRETIPLA
                        (CDFILIAL, NRINVRETIPLA, DTINIRETIPLA, DTFINRETIPLA, DSINVENTRETI, IDRETIEFET, NRORGINCLUSAO, CDOPERINCLUSAO, DTINCLUSAO)
                    VALUES
                        (:CDFILIAL, :NRINVRETIPLA, :DTINIRETIPLA, :DTFINRETIPLA, :DSINVENTRETI, :IDRETIEFET, :NRORG, :CDOPERADOR, SYSDATE)";
        $sequential = $this->getNewCode('INVENTRETIPLA'.$row['CDFILIAL']);
        $params = array(
            'CDFILIAL' => $row['CDFILIAL'],
            'NRINVRETIPLA' => $sequential,
            'DTINIRETIPLA' => $row['DTINIRETIPLA'],
            'DTFINRETIPLA' => $row['DTFINRETIPLA'],
            'DSINVENTRETI' => $row['DSINVENTRETI'],
            'IDRETIEFET' => 'N',
            'NRORG' => $this->environment->getNrOrg(), 
            'CDOPERADOR' =>  $this->environment->getCdOperador()
        );
         //coleta de dados
            $this->libUtilities->saveCollectedData($row['CDFILIAL'],'RETIRADA');
        $pstmt = $this->connection->prepare($query);
        $pstmt->execute($params);
        return $sequential;
    } 

    private function insereLancamento($row){
        $query = "INSERT INTO ITEMINVRETIPLA
                        (CDFILIAL, NRINVRETIPLA, NRITINVENTRETI, QTITINVENTRETI, CDPRODUTO, IDORIGLANCPRINV, NRORGINCLUSAO, CDOPERINCLUSAO, DTINCLUSAO)
                    VALUES
                        (:CDFILIAL, :NRINVRETIPLA, :NRITINVENTRETI, :QTITINVENTRETI, :CDPRODUTO, :IDORIGLANCPRINV, :NRORG, :CDOPERADOR, SYSDATE)";
        $sequential = $this->getNewCode('ITEMINVRETIPLA'.$row['CDFILIAL'], 1, 4);
        $params           = array(
            "CDFILIAL"        => $row['CDFILIAL'],
            "NRINVRETIPLA"    => $row['NRINVRETIPLA'],
            "NRITINVENTRETI"  => $sequential,
            "QTITINVENTRETI"  => $row['QTITINVENTRETI'],
            "CDPRODUTO"       => $row['CDPRODUTO'],
            "IDORIGLANCPRINV" => '0',
            'NRORG'           => $this->environment->getNrOrg(), 
            'CDOPERADOR'      => $this->environment->getCdOperador()
        );
         //coleta de dados
            $this->libUtilities->saveCollectedData($row['CDFILIAL'],'RETIRADA');
        $pstmt = $this->connection->prepare($query);
        $pstmt->execute($params);
    }
    
    private function updateLancamento($row) {
        $query = "UPDATE ITEMINVRETIPLA SET QTITINVENTRETI = :QTITINVENTRETI, NRORGULTATU = :NRORG, CDOPERULTATU = :CDOPERADOR, DTULTATU = SYSDATE WHERE CDFILIAL = :CDFILIAL AND NRINVRETIPLA = :NRINVRETIPLA AND NRITINVENTRETI = :NRITINVENTRETI AND NRORG = :NRORG";
        $params = array(
            'QTITINVENTRETI' => $row['QTITINVENTRETI'],
            'CDFILIAL' => $row['CDFILIAL'],
            'NRINVRETIPLA' => $row['NRINVRETIPLA'],
            'NRITINVENTRETI' => $row['NRITINVENTRETI'],
            'CDOPERADOR' => $this->environment->getCdOperador(),
            'NRORG' => $this->environment->getNrOrg()
        );
        //coleta de dados
	    $this->libUtilities->saveCollectedData($row['CDFILIAL'],'RETIRADA');
        $pstmt = $this->connection->prepare($query);
        $pstmt->execute($params);
    }

    public function saveProducts(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        //coleta de dados
	    $this->libUtilities->saveCollectedData($row['CDFILIAL'],'RETIRADA');
        $this->retiradaInventarioService->iniciaConexao();
        try{
            if(isset($row['__is_new']) && $row['__is_new']){
                $row['NRINVRETIPLA'] = $this->insereRomaneio($row);
            }
            if(isset($row['products']) && is_array($row['products']) && !empty($row['products'])){
                $products = $row['products'];
                $INSERT = $products['INSERT'];
                $UPDATE = $products['UPDATE'];
                $DELETE = $products['DELETE'];
                foreach($INSERT as $product){
                    $product['NRINVRETIPLA'] = $row['NRINVRETIPLA'];
                    $this->insereLancamento($product);
                }
                foreach($UPDATE as $product){
                    $this->updateLancamento($product);
                }
                foreach($DELETE as $product){
                    $this->deleteProduct($product);
                }
            }
            $this->retiradaInventarioService->realizaCommit();
        } catch(\Exception $e){
            $this->retiradaInventarioService->realizaRollback();
            $response->setError(new Response\Error($e->getMessage(), 1));
        }
    }
    
    public function deleteProducts(DTO\Request\DataSet $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        $firstProduct = $rows[0];
        $countProducts = $this->getCountProducts($firstProduct);
        try {
            $this->connection->beginTransaction();
            if(count($rows) == $countProducts){
                $firstProduct['NRITINVENTRETI'] = 'T';
                $this->deleteProduct($firstProduct);
            } else {
                foreach( $rows as $product ){
                    $this->deleteProduct($product);    
                }
            }
            $this->connection->commit();
        } catch (\Exception $e){
            $this->connection->rollback();
            $response->setError(new Response\Error($e->getMessage(), 1));
        }
    }
    
    private function getCountProducts ($row){
        $sql = "SELECT COUNT(*) AS TOTAL FROM ITEMINVRETIPLA WHERE CDFILIAL = :CDFILIAL AND NRINVRETIPLA = :NRINVRETIPLA";
        $params = array(
            'CDFILIAL' => $row['CDFILIAL'],
            'NRINVRETIPLA' => $row['NRINVRETIPLA']
        );
        $pstmt = $this->connection->prepare($sql);
        $pstmt->execute($params);
        return $pstmt->fetch()['TOTAL'];
    }
    
    private function deleteProduct($row){
        $sql = "DELETE FROM ITEMINVRETIPLA WHERE CDFILIAL = :CDFILIAL AND NRINVRETIPLA = :NRINVRETIPLA AND (NRITINVENTRETI = :NRITINVENTRETI OR :NRITINVENTRETI = 'T') AND NRORG = :NRORG";
        $params = array(
            'CDFILIAL'       => $row['CDFILIAL'],
            'NRINVRETIPLA'   => $row['NRINVRETIPLA'],
            'NRITINVENTRETI' => $row['NRITINVENTRETI'],
            'NRORG'          => $this->environment->getNrOrg()
        );
        $pstmt = $this->connection->prepare($sql);
        $pstmt->execute($params);
    }
    
    private function validateDeleteRomaneio($row){
        $sql = "SELECT IDRETIEFET FROM INVENTRETIPLA WHERE CDFILIAL = :CDFILIAL AND NRINVRETIPLA = :NRINVRETIPLA AND NRORG = :NRORG";
        $params = array(
            'CDFILIAL' => $row['CDFILIAL'],
            'NRINVRETIPLA' => $row['NRINVRETIPLA'],
            'NRORG' => $this->environment->getNrOrg()
        );
        $pstmt = $this->connection->prepare($sql);
        $romaneio = $pstmt->fetch();
        if($romaneio){
            if($romaneio['IDRETIEFET'] == 'S'){
                throw new \Exception("MSG_PLA_ROMANEIO_COM_RETIRADA");
            }
        }
    }
    
    public function deleteRomaneio(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        try {
            $this->connection->beginTransaction();
            
            $this->validateDeleteRomaneio($row);
            
            $row['NRITINVENTRETI'] = 'T';
            $this->deleteProduct($row);
            
            $sql = "DELETE FROM INVENTRETIPLA WHERE CDFILIAL = :CDFILIAL AND NRINVRETIPLA = :NRINVRETIPLA AND NRORG = :NRORG";
            $params = array(
                'CDFILIAL'     => $row['CDFILIAL'],
                'NRINVRETIPLA' => $row['NRINVRETIPLA'],
                'NRORG'        => $this->environment->getNrOrg()
            );
            $pstmt = $this->connection->prepare($sql);
            $pstmt->execute($params);
            $this->connection->commit();
        } catch (\Exception $e){
            $this->connection->rollback();
            $response->setError(new Response\Error($e->getMessage(), 1));
        }
    }

    public function isValidPeriod(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $query = "SELECT CDFILIAL, NRINVRETIPLA, DTINIRETIPLA, DTFINRETIPLA FROM INVENTRETIPLA 
                    WHERE (((( DTINIRETIPLA <= :P_DTINIRETIPLA ) AND ( DTFINRETIPLA >= :P_DTINIRETIPLA )) OR 
                        (( DTINIRETIPLA <= :P_DTFINRETIPLA ) AND ( DTFINRETIPLA >= :P_DTFINRETIPLA ))) OR 
                        (( DTINIRETIPLA >= :P_DTINIRETIPLA ) AND ( DTFINRETIPLA <= :P_DTFINRETIPLA ))) AND
                        CDFILIAL = :P_CDFILIAL";
        $params = array(
            "P_DTINIRETIPLA" => $row['DTINIRETIPLA'],
            "P_DTFINRETIPLA" => $row['DTFINRETIPLA'],
            "P_CDFILIAL" => $row['CDFILIAL']
        );    
        $pstmt = $this->connection->prepare($query);
        $pstmt->execute($params);
        $result = $pstmt->fetchAll();
        $response->addDataSet(new DataSource\DataSet("period", $result));
    }

    private function getSequentialRomaneio($row){
        $query = "SELECT NVL(LPAD(MAX(TO_NUMBER(NRINVRETIPLA)+1), 10, '0'), LPAD('0',10,'1')) AS SEQ_PROX FROM INVENTRETIPLA WHERE CDFILIAL = :CDFILIAL AND NRORG = :NRORG";
        $params = array(
            'CDFILIAL' => $row['CDFILIAL'],
            'NRORG'    => $this->environment->getNrOrg()
        );
        $pstmt = $this->connection->prepare($query);
        $pstmt->execute($params);
        return $pstmt->fetchAll()[0]['SEQ_PROX'];
    }

    private function getSequentialLancamento($row){
        $query = "SELECT LPAD(NVL(MAX(TO_NUMBER(NRITINVENTRETI)+1), '1'), 10, '0') AS SEQ_PROX FROM ITEMINVRETIPLA WHERE CDFILIAL = :CDFILIAL AND NRINVRETIPLA = :NRINVRETIPLA AND NRORG = :NRORG";
        $params = array(
            'CDFILIAL' => $row['CDFILIAL'],
            'NRINVRETIPLA' => $row['NRINVRETIPLA'],
            'NRORG' => $this->environment->getNrOrg()
        );
        $pstmt = $this->connection->prepare($query);
        $pstmt->execute($params);
        return $pstmt->fetchAll()[0]['SEQ_PROX'];
    }

    private function getNewCode($param, $interval = 1, $sizeColumn = 10){
        $nrorg = $this->environment->getNrOrg();
        return $this->newCode->getCodeByTableName($param, $nrorg, $interval, $sizeColumn);
    }
    
    public function retiradaAutomatica (DTO\Request\Row $row, DTO\Response $response){
        $row = $row->getRow();
         //coleta de dados
        $this->libUtilities->saveCollectedData($row['CDFILIAL'],'RETIRADA');
        $erroAtualizarEstoque = array();
        try {
            $this->retiradaInventarioService->iniciaConexao();
            $result = $this->retiradaInventarioService->executaProcedureRetiradaInventarioPla($row);       
            if($result && isset($result['P_RETORNO']) && $result['P_RETORNO'] == 'SUCESSO'){
                $this->retiradaInventarioService->realizaCommit();
                $retiradasGeradas = $this->retiradaInventarioService->retornaRetiradasPeriodo($row);
                foreach($retiradasGeradas as $retirada){
                    try {
                        $this->retiradaInventarioService->executaAtualizacaoEstoque($retirada);
                    } catch (\Exception $e){
                        $this->trataRetiradaErro($retirada, $erroAtualizarEstoque, $e->getMessage());
                    }
                }
                $result['ERRORS_ESTOQUE'] = $erroAtualizarEstoque;
            } else {
                $this->retiradaInventarioService->realizaRollback();    
                if(isset($result['P_RETORNO']) && $result['P_RETORNO'] == 'ERRO'){
                    if(isset($result['P_LISTAERROS']) && $result['P_LISTAERROS']){
                        
                        $errorList = $this->retiradaInventarioService->getMessageErrorList($row['CDFILIAL'], $result['P_LISTAERROS']);
                        $formatErrors = array();
                        foreach($errorList as $errorProcedure){
                            array_push($formatErrors, $this->retiradaInventarioService->converter($errorProcedure['DSERRO']));
                        }
                        $result['ERRORS'] = $formatErrors;
                    }
                }
                
            }
            $response->addDataSet(new DataSource\DataSet("RESULT_PROCEDURE", $result));
        } catch (\Exception $e){
            $this->retiradaInventarioService->realizaRollback();
            $response->setError(new Response\Error($e->getMessage(), 1));
        }
    }
    
    private function trataRetiradaErro($retirada, &$dataSetError, $msgError){
        if(!isset($dataSetError[$retirada['CDSERVFILI']])){
            $dataSetError[$retirada['CDSERVFILI']] = array(
                'NMSERVICO' => $retirada['NMSERVICO'],
                'msgError' => $msgError,
                'DTRETIRADA' => array($retirada['DTRETIRADA'])
            );
        } else {
            array_push($dataSetError[$retirada['CDSERVFILI']]['DTRETIRADA'], $retirada['DTRETIRADA']);
        }
    }

}