<?php
namespace Teknisa\PLA\Modules\Prato\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet as DataSet_Source;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;
use Doctrine\DBAL\Connection;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class CargaFotoPratoController {
    
    protected $fileServerURL;
    protected $crudService;
    protected $environment;
    protected $connection;
    protected $blobToFileServer;
    protected $nrlista;
    protected $entityManager;

    public function __construct($blobToFileServer,$fileServerURL,$entityManager,$crudService,$environment,$connection) {
        $this->blobToFileServer = $blobToFileServer;
        $this->fileServerURL   = $fileServerURL;
        $this->entityManager   = $entityManager;
        $this->crudService     = $crudService;
        $this->environment     = $environment;
        $this->connection      = $connection;
    }
    
    public function retornaQuantPratosFotoDesatualizada(DTO\Request $request, DTO\Response $response){
        try {
            $sql = "SELECT  FPB.NRFOTOPRATOBLOB,
                            FPB.CDFILAUXPRAT,
                            FPB.CDPRATO,
                            FPB.IDRECEITA,
                            FPB.CDFILIAL,
                            FPB.CDSERVFILI,
                            FPB.NRFOTO,
                            FPB.FGFOTOPRATOURL 
                    FROM FOTOPRATOBLOB FPB
                    LEFT JOIN PRATO P ON P.CDPRATO = FPB.CDPRATO
                                    AND P.CDFILAUXPRAT = FPB.CDFILAUXPRAT
                    WHERE FPB.FGFOTOPRATO IS NOT NULL 
                        AND FPB.FGFOTOPRATOURL IS NULL
                        AND P.CDNVPRATO = '4'";
            $parameter = array();
            $pratos = $this->entityManager->getConnection()->fetchAll($sql, $parameter);
            $response->addDataset(new DataSet_Source('RETORNO', $pratos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function updateFotoPrato(DTO\Request $request, DTO\Response $response){
        try {
            
            $rows = $request->getRow()['PRATOS'];

            foreach ($rows as $prato) {
                
                $sql = "SELECT FGFOTOPRATO FROM FOTOPRATOBLOB 
                        WHERE NRFOTOPRATOBLOB = :NRFOTOPRATOBLOB";
                $parameter = array(
                    "NRFOTOPRATOBLOB" => $prato['NRFOTOPRATOBLOB']
                );
                $foto = $this->entityManager->getConnection()->fetchAll($sql, $parameter);
                
                $nomeCampo = 'PHOTO_UPLOAD';
                $customFolder = 'Planejamento/Prato';
                $nomeFoto = null;
                
                switch ($prato['IDRECEITA']) {
                    case "P":
                        $nomeFoto = $prato['CDPRATO'];
                        break;
                    case "F":
                        $nomeFoto = $prato['CDPRATO'].'_'.$prato['CDFILIAL'] ;
                        break;
                    case "S":
                        $nomeFoto = $prato['CDPRATO'].'_'.$prato['CDFILIAL'].'_'.$prato['CDSERVFILI'];
                        break;
                }
                
                $fotoConvertida = $this->blobToFileServer->converteESalvaBlobParaFileServer($foto[0]['FGFOTOPRATO'], $nomeCampo, $nomeFoto, $customFolder);  
                
                if (isset($fotoConvertida[$nomeCampo][0]['path'])) {
                    
                    $dadosUpdate = array(
                        "NRFOTOPRATOBLOB" => $prato['NRFOTOPRATOBLOB'],
                        "FGFOTOPRATOURL" => $fotoConvertida[$nomeCampo][0]['path']
                    );
                
                    $this->updateFgfotopratourl($dadosUpdate);
                }
                
            }
            $response->addDataset(new DataSet_Source('SUCESSO'));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function updateFgfotopratourl($dadosUpdate){
        try {
            $this->connection->beginTransaction();
            $update = "UPDATE FOTOPRATOBLOB SET FGFOTOPRATOURL = :FGFOTOPRATOURL
                        WHERE NRFOTOPRATOBLOB = :NRFOTOPRATOBLOB";

            $params = array(
               'FGFOTOPRATOURL' => $dadosUpdate['FGFOTOPRATOURL'],
               'NRFOTOPRATOBLOB' => $dadosUpdate['NRFOTOPRATOBLOB']
            );
            
            $query = $this->connection->prepare($update);
            $query->execute($params);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
        }
        
    }

}