<?php
namespace Teknisa\PLA\Modules\Prato\Controllers;


use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet as DataSet_Source;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;
use Doctrine\DBAL\Connection;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Teknisa\PLA\Util\RtfToHtml\Document;
use Teknisa\PLA\Util\RtfToHtml\HtmlFormatter;
class CargaModoPreparoController {
    protected $crudService;
    protected $environment;
    protected $connection;
    protected $util;
    protected $nrlista;
    protected $entityManager;

    public function __construct($entityManager,$crudService,$environment,$connection,$util) {
        $this->entityManager                        = $entityManager;
        $this->crudService                          = $crudService;
        $this->environment                          = $environment;
        $this->connection                           = $connection;
        $this->util                                 = $util;
    }
    
    public function retornaQuantPratosModoPrepDesatualizado(DTO\Request $request, DTO\Response $response){
        try {
            $sql = "SELECT *
                    FROM PRATO P
                    WHERE P.CDNVPRATO = '4'
                    AND EXISTS
                    (
                            SELECT 1
                            FROM PREPPRASER
                            WHERE TXPREPSERV IS NOT NULL
                              AND TXPREPHTML IS NULL
                              AND CDFILAUXPRAT = P.CDFILAUXPRAT
                              AND CDPRATO = P.CDPRATO
                    
                            UNION ALL
                            
                            SELECT 1
                            FROM PREPPRAFIL
                            WHERE TXPREPFILI IS NOT NULL
                              AND TXPREPHTML IS NULL
                              AND CDFILAUXPRAT = P.CDFILAUXPRAT
                              AND CDPRATO = P.CDPRATO
                            UNION ALL
                            
                            SELECT 1
                            FROM PREPPRAPAD 
                            WHERE TXPREPPRAT IS NOT NULL
                              AND TXPREPHTML IS NULL
                              AND CDFILAUXPRAT = P.CDFILAUXPRAT
                              AND CDPRATO = P.CDPRATO)";
            $parameter = array();
            $pratos = $this->entityManager->getConnection()->fetchAll($sql, $parameter);
            $response->addDataset(new DataSet_Source('RETORNO', $pratos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function retornaModoPreparo($prato){
        try {
            $sql = "SELECT MODOS.CDFILIAL,
                           MODOS.CDSERVFILI,
                           MODOS.CDFILAUXPRAT,
                           MODOS.CDPRATO,
                           MODOS.NR_PREPARO,
                           MODOS.ID_PREPARO,
                           MODOS.CAMPO_RECEITA_ANTIGO,
                           MODOS.TXPREPHTML
                    FROM (
                        SELECT CDFILIAL,
                               CDSERVFILI,
                               CDFILAUXPRAT,
                               CDPRATO,
                               NRPREPSERV AS NR_PREPARO,
                               IDPREPSERV AS ID_PREPARO,
                               TXPREPSERV AS CAMPO_RECEITA_ANTIGO,
                               TXPREPHTML
                        FROM PREPPRASER
                        WHERE TXPREPSERV IS NOT NULL
                          AND TXPREPHTML IS NULL
                
                        UNION ALL
                
                        SELECT CDFILIAL,
                               NULL AS CDSERVFILI,
                               CDFILAUXPRAT,
                               CDPRATO,
                               NRPREPFILI AS NR_PREPARO,
                               IDPREPFILI AS ID_PREPARO,
                               TXPREPFILI AS CAMPO_RECEITA_ANTIGO,
                               TXPREPHTML
                        FROM PREPPRAFIL
                        WHERE TXPREPFILI IS NOT NULL
                          AND TXPREPHTML IS NULL
                
                        UNION ALL
                
                        SELECT NULL AS CDFILIAL,
                               NULL AS CDSERVFILI,
                               CDFILAUXPRAT,
                               CDPRATO,
                               NRPREPPRAT AS NR_PREPARO,
                               IDPREPPRAT AS ID_PREPARO,
                               TXPREPPRAT AS CAMPO_RECEITA_ANTIGO,
                               TXPREPHTML
                        FROM PREPPRAPAD 
                        WHERE TXPREPPRAT IS NOT NULL
                          AND TXPREPHTML IS NULL
                    ) MODOS
                WHERE     MODOS.CDPRATO = :CDPRATO
                          AND MODOS.CDFILAUXPRAT = :CDFILAUXPRAT";
            $parameter = array(
        		    "CDPRATO" => $prato['CDPRATO'],
        		    "CDFILAUXPRAT" => $prato['CDFILAUXPRAT']
                );
            $modos = $this->entityManager->getConnection()->fetchAll($sql, $parameter);
            return $modos;
        } catch (\Exception $e) {
            
        }
    }
    
    public function updateModoReceita(DTO\Request $request, DTO\Response $response){
        try {
            $rows = $request->getRow()['PRATOS'];

            foreach ($rows as $prato) {
                
                $modos = $this->retornaModoPreparo($prato);
                
                foreach ($modos as $modo) {
                    $receita;
                    if ($modo) {
                        $receita = "P";
                        if ($modo['CDFILIAL'] !== null) {
                            $receita = "L";
                        }
                        if ($modo['CDSERVFILI'] !== null) {
                            $receita = "S";
                        }
                    }
                   
                    $modoConvertidoHtml = $this->converteModosPrepHtml($modo);
               
                    switch ($receita) {
                        case "P":
                            $this->updateModoReceitaPadrao($modoConvertidoHtml);
                            break;
                        case "L":
                            $this->updateModoReceitaFilial($modoConvertidoHtml);
                            break;
                        case "S":
                            $this->updateModoReceitaServico($modoConvertidoHtml);
                            break;
                    }
                }
            }

            $response->addDataset(new DataSet_Source('SUCESSO'));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function converteModosPrepHtml ($modosPrep){
         
        if(!empty($modosPrep['CAMPO_RECEITA_ANTIGO']) && empty($modosPrep['TXPREPHTML'])){
            $modosPrep['TXPREPHTML'] = utf8_encode(str_replace(chr(0), "",$modosPrep['CAMPO_RECEITA_ANTIGO']));

            if($this->isRtf($modosPrep['TXPREPHTML'])){
                $formatter = new HtmlFormatter('UTF-8');
                $document = new Document($modosPrep['TXPREPHTML']);
                $modosPrep['TXPREPHTML'] = $formatter->Format($document);
            }
        }

        return $modosPrep;
    }
    
    function isRtf($str){
        
        if(empty($str)) {
            return false;
        } else if (strpos($str, "rtf") !== false) {
            return true;
        }
       return false;
    }
    
    public function updateModoReceitaPadrao ($modoPrep){
        try {
            $this->connection->beginTransaction();
            $update = "UPDATE PREPPRAPAD SET TXPREPHTML = :TXPREPHTML
                       WHERE CDPRATO = :CDPRATO
                            AND CDFILAUXPRAT = :CDFILAUXPRAT
                            AND NRPREPPRAT = :NRPREPPRAT
                            AND IDPREPPRAT = :IDPREPPRAT";

            $params = array(
               'CDPRATO' => $modoPrep['CDPRATO'],
               'CDFILAUXPRAT' => $modoPrep['CDFILAUXPRAT'],
               'NRPREPPRAT' => $modoPrep['NR_PREPARO'],
               'IDPREPPRAT' => $modoPrep['ID_PREPARO'],
               'TXPREPHTML' => $modoPrep['TXPREPHTML']
            );
            
            $query = $this->connection->prepare($update);
            $query->execute($params);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
        }
        
    }
    
    public function updateModoReceitaFilial ($modoPrep){
        try {
            $this->connection->beginTransaction();
            $update = "UPDATE PREPPRAFIL SET TXPREPHTML = :TXPREPHTML
                       WHERE CDPRATO = :CDPRATO
                            AND CDFILAUXPRAT = :CDFILAUXPRAT
                            AND NRPREPFILI = :NRPREPFILI
                            AND IDPREPFILI = :IDPREPFILI
                            AND CDFILIAL = :CDFILIAL";

            $params = array(
               'CDPRATO' => $modoPrep['CDPRATO'],
               'CDFILAUXPRAT' => $modoPrep['CDFILAUXPRAT'],
               'NRPREPFILI' => $modoPrep['NR_PREPARO'],
               'IDPREPFILI' => $modoPrep['ID_PREPARO'],
               'CDFILIAL' => $modoPrep['CDFILIAL'],
               'TXPREPHTML' => $modoPrep['TXPREPHTML']
            );
        
            $query = $this->connection->prepare($update);
            $query->execute($params);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
        }
      
    }
    
    public function updateModoReceitaServico ($modoPrep){
        try {
            $this->connection->beginTransaction();
            $update = "UPDATE PREPPRASER SET TXPREPHTML = :TXPREPHTML
                       WHERE CDPRATO = :CDPRATO
                            AND CDFILAUXPRAT = :CDFILAUXPRAT
                            AND NRPREPSERV = :NRPREPSERV
                            AND IDPREPSERV = :IDPREPSERV
                            AND CDFILIAL = :CDFILIAL
                            AND CDSERVFILI = :CDSERVFILI";

            $params = array(
               'CDPRATO' => $modoPrep['CDPRATO'],
               'CDFILAUXPRAT' => $modoPrep['CDFILAUXPRAT'],
               'NRPREPSERV' => $modoPrep['NR_PREPARO'],
               'IDPREPSERV' => $modoPrep['ID_PREPARO'],
               'CDFILIAL' => $modoPrep['CDFILIAL'],
               'CDSERVFILI' => $modoPrep['CDSERVFILI'],
               'TXPREPHTML' => $modoPrep['TXPREPHTML']
            );
            
            $query = $this->connection->prepare($update);
            $query->execute($params);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
        }   
        
    }
   
}