<?php
namespace Teknisa\PLA\Modules\Relatorio\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;

class Pla07900DiferencaDeComensais extends \Zeedhi\Framework\Controller\Report {

    protected $qr2Strategy;
    protected $birtStrategy;
    protected $entityManager;
    protected $novoCodigo;
    protected $crudService;

    public function __construct($reportService, $qr2Strategy, $birtStrategy, $entityManager, $novoCodigo, $crudService) {

        $this->qr2Strategy = $qr2Strategy;
        $this->birtStrategy = $birtStrategy;
        $this->entityManager = $entityManager;
        $this->novoCodigo = $novoCodigo;
        $this->crudService = $crudService;

        parent::__construct($reportService);
    }

    protected $reportMapping = array (
    'pla07900' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDGRUPFILI'  => 'P_CDGRUPFILI',
                'LISTASERV'   => 'P_NRLISTASERV',
                'DTMESINIC'   => 'P_DTINI',
                'DTMESFINA'   => 'P_DTFIN',
                'NRORG'       => 'P_NRORG'
            ),
        ),
    'pla07901' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDGRUPFILI'  => 'P_CDGRUPFILI',
                'LISTASERV'   => 'P_NRLISTASERV',
                'DTMESINIC'   => 'P_DTINI',
                'DTMESFINA'   => 'P_DTFIN',
                'NRORG'       => 'P_NRORG'
            ),
        ),
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {

        $row = $request->getRow();
        if(empty($row['CDGRUPFILI'])) $row['CDGRUPFILI'] = null;

        $row['CDFILIAL1'] = $row['LISTAFILIAL'];

        if($row['ALLCDSERVFILI'] == 'T') $row['NMSERVICO'] = $this->getServicos($row);
        $row['LISTASERV'] = $this->geraListaServico($row['NMSERVICO'], $row['NRORG']);

        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
    }

    private function getServicos($row){
        $cdfilial = $row['ALLCDFILIAL'] == 'T'? 'T' : implode('_', $row['NMFILIAL']);

        $dataSourceName = 'servicos_por_agrupfili_multi_filiais';
        
        $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator"   => '=',
                "value"      => $cdfilial
            ),
            array(
                "columnName" => "IDTIPOSERV",
                "operator"   => '=',
                "value"      => 'N'
            ),
            array(
                "columnName" => "CDOPERADOR",
                "operator"   => '=',
                "value"      => $row['CDOPERADOR']
            ),
            array(
                "columnName" => "CDGRUPFILI",
                "operator"   => '=',
                "value"      => $row['CDGRUPFILI'] ? $row['CDGRUPFILI'] : 'T'
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
        return $dataSet->getRows();
    }

    private function geraListaServico($filiserv, $nrorg){
        
        $nrlistaServico = $this->novoCodigo->getCodeByTableName('LISTA_SERVICO', $nrorg);
        
        $sql = "INSERT INTO LISTA_SERVICO (NRLISTA, CDFILIAL, CDSERVFILI, NMSERVICO, CDGRUPSERV, NRORG) VALUES (:NRLISTA, :CDFILIAL, :CDSERVFILI, :NMSERVICO, :CDGRUPSERV, :NRORG)";
        
        foreach ($filiserv as $servicoFilial) {
            if(!empty($servicoFilial)){                
                $stmt = $this->entityManager->getConnection()->prepare($sql);
                $stmt->execute(
                    array(
                        "NRLISTA" => $nrlistaServico,
                        "CDFILIAL" => $servicoFilial['CDFILIAL'],
                        "NRORG" => $nrorg,
                        "CDSERVFILI" => $servicoFilial['CDSERVFILI'],
                        "NMSERVICO" => $servicoFilial['NMSERVICO'],
                        "CDGRUPSERV" => null
                    )
                );
                
            }
            
        }
        
        return $nrlistaServico;
    }

}